// This MFC Samples source code demonstrates using MFC Microsoft Office Fluent User Interface 
// (the "Fluent UI") and is provided only as referential material to supplement the 
// Microsoft Foundation Classes Reference and related electronic documentation 
// included with the MFC C++ library software.  
// License terms to copy, use or distribute the Fluent UI are available separately.  
// To learn more about our Fluent UI licensing program, please visit 
// http://go.microsoft.com/fwlink/?LinkId=238214.
//
// Copyright (C) Microsoft Corporation
// All rights reserved.

// CodeEditorDoc.cpp : implementation of the CCodeEditorDoc class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "CodeEditor.h"
#endif

#include "CodeEditorDoc.h"
#include "CntrItem.h"

#include <propkey.h>

#include "CodeEditorView.h"

#include "DlgAnsiFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CCodeEditorDoc

IMPLEMENT_DYNCREATE(CCodeEditorDoc, CRichEditDoc)

BEGIN_MESSAGE_MAP(CCodeEditorDoc, CRichEditDoc)
	// Enable default OLE container implementation
	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_LINKS, &CRichEditDoc::OnUpdateEditLinksMenu)
	ON_UPDATE_COMMAND_UI(ID_OLE_VERB_POPUP, &CCodeEditorDoc::OnUpdateObjectVerbPopup)
	ON_COMMAND(ID_OLE_EDIT_LINKS, &CRichEditDoc::OnEditLinks)
	ON_UPDATE_COMMAND_UI_RANGE(ID_OLE_VERB_FIRST, ID_OLE_VERB_LAST, &CRichEditDoc::OnUpdateObjectVerbMenu)
END_MESSAGE_MAP()


// CCodeEditorDoc construction/destruction

CCodeEditorDoc::CCodeEditorDoc()
{
	// TODO: add one-time construction code here

	// instructs to save all text as ANSI text
	CRichEditDoc::m_bRTF = FALSE;
}

CCodeEditorDoc::~CCodeEditorDoc()
{
}

BOOL CCodeEditorDoc::OnNewDocument()
{
	if (!CRichEditDoc::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

CRichEditCntrItem* CCodeEditorDoc::CreateClientItem(REOBJECT* preo) const
{
	return new CCodeEditorCntrItem(preo, const_cast<CCodeEditorDoc*>(this));
}

// CCodeEditorDoc serialization

void CCodeEditorDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here

		CString text;
		CCodeEditorView* pView
			= reinterpret_cast<CCodeEditorView*>(this->m_viewList.GetHead());

		pView->GetWindowTextW(text);

		// UTF-16 Byte Order Mark for Little Endian System
		unsigned short bom = 0xFEFF;

		ar.Write(&bom, sizeof(bom));

		ar.WriteString(text);

	}
	else // loading contents
	{
		// TODO: add loading code here

		if (!this->m_viewList.IsEmpty())
		{
			unsigned short bom;
			ar >> bom;

			if (bom == 0xFEFF) // utf-16
			{
				CFile* pFile = ar.GetFile();
				// -2 because we've ready read bom, 2 bytes
				ULONGLONG nFileSizeBytes = pFile->GetLength() - 2;
				ULONGLONG nFileSizeChars = nFileSizeBytes / sizeof(wchar_t);

				// create buffer to read in characters
				// +1 for null character
				std::unique_ptr<wchar_t[]>
					lpszText(new wchar_t[size_t(nFileSizeChars) + 1]);
				
				if (lpszText)
				{
					ar.Read(lpszText.get(), (UINT)nFileSizeBytes);
					lpszText[size_t(nFileSizeChars)] = L'\0';

					CCodeEditorView* pView
						= reinterpret_cast<CCodeEditorView*>(this->m_viewList.GetHead());

					pView->SetWindowTextW(lpszText.get());

				}

			}
			else // not utf-16
			{
				CFile* pFile = ar.GetFile();
				pFile->SeekToBegin();

				unsigned char utf8_bom[3];

				pFile->Read(&utf8_bom, 3);

				// utf-8 encoded
				if ((utf8_bom[0] == 0xEF) &&
					(utf8_bom[1] == 0xBB) &&
					(utf8_bom[2] == 0xBF))
				{
					// because we already read 3 bytes for bom
					ULONGLONG nFileSizeBytes = pFile->GetLength() - 3;

					// + 1 for null character
					std::unique_ptr<char[]>
						lpszText(new char[size_t(nFileSizeBytes) + 1]);

					if (lpszText)
					{
						pFile->Read(lpszText.get(), (UINT)nFileSizeBytes);
						lpszText[size_t(nFileSizeBytes)] = '\0';

						// convert utf8 to utf-16
						CString txt = siliners::utf8_to_cstr(lpszText.get());
						lpszText.reset();

						CCodeEditorView* pView
							= reinterpret_cast<CCodeEditorView*>(this->m_viewList.GetHead());

						if (pView)
							pView->SetWindowTextW(txt);

					}

				}
				else  // neither UCS nor UTF8
				{
					// ANSI multibyte text

					CFile *pFile = ar.GetFile();
					pFile->SeekToBegin();
					ULONGLONG nFileSizeBytes = pFile->GetLength();

					// +1 for null character
					std::unique_ptr<char[]>
						lpszText(new char[size_t(nFileSizeBytes) + 1]);

					if (lpszText)
					{
						pFile->Read(lpszText.get(), (UINT)nFileSizeBytes);
						lpszText[size_t(nFileSizeBytes)] = '\0';

						const size_t short_line = 50;

						// +1 for null character
						std::unique_ptr<char[]> short_text(new char[short_line + 1]);

						memcpy_s(short_text.get(),
							short_line, lpszText.get(), short_line);

						if (short_line < size_t(nFileSizeBytes))
							short_text[short_line] = '\0';
						else
							short_text[size_t(nFileSizeBytes)] = '\0';

						CDlgAnsiFile dlg;
						dlg.m_str = short_text.get();
												
						if (dlg.DoModal() == IDOK)
						{
							int index = int(dlg.m_Index);

							CString txt;

							std::vector<std::pair<std::wstring, unsigned int>>
								code_page_vctr = siliners::get_code_page_vctr();

							txt = siliners::mbs_to_cstr(lpszText.get(),
								code_page_vctr[index].second);

							lpszText.reset();

							if (txt.GetLength() > 0)
							{
								CCodeEditorView* pView
									= reinterpret_cast<CCodeEditorView*>
									(this->m_viewList.GetHead());

								if (pView) pView->SetWindowTextW(txt);
							}

						}					
					}
				}
			}
		}
	}

	// Calling the base class CRichEditDoc enables serialization
	//  of the container document's COleClientItem objects.
	// TODO: set CRichEditDoc::m_bRTF = FALSE if you are serializing as text
	// CRichEditDoc::Serialize(ar);
}

#ifdef SHARED_HANDLERS

// Support for thumbnails
void CCodeEditorDoc::OnDrawThumbnail(CDC& dc, LPRECT lprcBounds)
{
	// Modify this code to draw the document's data
	dc.FillSolidRect(lprcBounds, RGB(255, 255, 255));

	CString strText = _T("TODO: implement thumbnail drawing here");
	LOGFONT lf;

	CFont* pDefaultGUIFont = CFont::FromHandle((HFONT) GetStockObject(DEFAULT_GUI_FONT));
	pDefaultGUIFont->GetLogFont(&lf);
	lf.lfHeight = 36;

	CFont fontDraw;
	fontDraw.CreateFontIndirect(&lf);

	CFont* pOldFont = dc.SelectObject(&fontDraw);
	dc.DrawText(strText, lprcBounds, DT_CENTER | DT_WORDBREAK);
	dc.SelectObject(pOldFont);
}

// Support for Search Handlers
void CCodeEditorDoc::InitializeSearchContent()
{
	CString strSearchContent;
	// Set search contents from document's data. 
	// The content parts should be separated by ";"

	// For example:  strSearchContent = _T("point;rectangle;circle;ole object;");
	SetSearchContent(strSearchContent);
}

void CCodeEditorDoc::SetSearchContent(const CString& value)
{
	if (value.IsEmpty())
	{
		RemoveChunk(PKEY_Search_Contents.fmtid, PKEY_Search_Contents.pid);
	}
	else
	{
		CMFCFilterChunkValueImpl *pChunk = NULL;
		ATLTRY(pChunk = new CMFCFilterChunkValueImpl);
		if (pChunk != NULL)
		{
			pChunk->SetTextValue(PKEY_Search_Contents, value, CHUNK_TEXT);
			SetChunkValue(pChunk);
		}
	}
}

#endif // SHARED_HANDLERS

// CCodeEditorDoc diagnostics

#ifdef _DEBUG
void CCodeEditorDoc::AssertValid() const
{
	CRichEditDoc::AssertValid();
}

void CCodeEditorDoc::Dump(CDumpContext& dc) const
{
	CRichEditDoc::Dump(dc);
}
#endif //_DEBUG


// CCodeEditorDoc commands
